** Demo04_dates.sas;

data dates;
  length transID 8 textdate $30. SAS_internal $25 MS_date_val 8;
  input transID $ date : mmddyy10.;
  textdate = put(date,worddate30.);
  SAS_internal = catx(' ','Internal Number:',put(date,best8.));
  MS_date_val = date + 21916;
  SAS_date9 = date;
  format SAS_date9 date9.;
return;
datalines;
1 11/15/1950
2 01/01/1960
3 11/29/1984
4 12/05/2005
5 01/20/2011
;
run;

ods listing;
options ls=125;
proc print data=dates noobs;
format transID best12. MS_date_val best12.;
run;

proc contents data=dates;
run;
                 
title;
footnote;
ods listing close;
ods tagsets.msoffice2k file='demo04_default_mso.xls' 
    style=sasweb;
ods tagsets.excelxp file='demo04_default_XP.xls' 
    style=sasweb;
proc report data=dates nowd
  style(column)={width=1.5in};
  title '1) Default for Dates';
  column transID textdate SAS_internal MS_date_val
         date SAS_date9;
  define transID /display style(column)={width=.75in};
  define textdate / 'Character Worddate';
  define SAS_internal / display 'Stored in SAS';
  define MS_date_val / 'Microsoft Internal Value/SAS val + 21916'
         style(column)={width=2in};
  define date / display 'NO SAS Fmt for Date/Excel shows internal number';
  define SAS_date9 /display  f=date9. 'SAS Date9. Format/Used Explicitly';
run;
ods _all_ close;

ods tagsets.msoffice2k file='demo04a_dates_mso.xls' style=sasweb;
proc report data=dates nowd
  style(column)={width=1.5in};
  title '2a) HTMLSTYLE for Dates -- Still Problem';
  column transID textdate SAS_internal MS_date_val
         date SAS_date9;
  define transID /display style(column)={width=.75in};
  define textdate / 'Character Worddate';
  define SAS_internal / display 'Stored in SAS';
  define MS_date_val / 'Microsoft Internal Value/SAS val + 21916'
         style(column)={width=2in
                        htmlstyle="mso-number-format:mm\/dd\/yyyy"};
  define date / display 'NO SAS Fmt for Date/Shows Date is 60 yrs "off"'
         style(column)={htmlstyle="mso-number-format:mm\/dd\/yyyy"};
  define SAS_date9 /display  f=date9. 'SAS Date9 Format/Used Explicitly'
         style(column)={htmlstyle="mso-number-format:mm\/dd\/yyyy"};
run;
ods _all_ close;

ods tagsets.msoffice2k file='demo04b_dates_mso.xls' style=sasweb;
proc report data=dates nowd
  style(column)={width=1.5in};
  title '2b)Fix HTMLSTYLE for All Dates';
  column transID textdate SAS_internal MS_date_val
         date SAS_date9;
  define transID /display style(column)={width=.75in};
  define textdate / 'Character Worddate'
         style(column)={htmlstyle="mso-number-format:\@"};
  define SAS_internal / display 'Stored in SAS';
  define MS_date_val / 'Microsoft Internal Value/SAS val + 21916'
         style(column)={width=2in
                        htmlstyle="mso-number-format:mm\/dd\/yyyy"};
  define date /display f=date9. 'With SAS Fmt for Date/Shows OK'
         style(column)={htmlstyle="mso-number-format:mm\-dd\-yyyy"};
  define SAS_date9 /display  f=date9. 'SAS Date9 Format/Used Explicitly'
         style(column)={width=2in
         htmlstyle="mso-number-format:mm\.dd\.yyyy"};
run;
ods _all_ close;

ods tagsets.excelxp file='demo04a_dates_XP.xls' 
    style=sasweb;
proc report data=dates nowd
  style(column)={width=1.5in};
  title '3a) TAGATTR for Dates -- Still Problem';
  column transID textdate SAS_internal MS_date_val
         date SAS_date9;
  define transID /display style(column)={width=.75in};
  define textdate / 'Character Worddate';
  define SAS_internal / display 'Stored in SAS';
  define MS_date_val / 'Microsoft Internal Value/SAS val + 21916'
         style(column)={width=2in
                        tagattr="format:mm/dd/yyyy"};
  define date / display 'NO SAS Fmt for Date/Shows Date is 60 yrs "off"'
         style(column)={tagattr="format:mm/dd/yyyy"};
  define SAS_date9 /display  f=date9. 'SAS Date9 Format/Used Explicitly'
         style(column)={tagattr="format:mm/dd/yyyy"};
run;

ods _all_ close;

title;
footnote;

ods tagsets.excelxp file='demo04b_dates_XP.xls' 
    style=sasweb;
proc report data=dates nowd
  style(column)={width=1.5in};
  title '3b) In 9.2 ExcelXP sends SAS formatted dates as a String';
  column transID textdate SAS_internal MS_date_val
         date SAS_date9;
  define transID /display style(column)={width=.75in};
  define textdate / 'Character Worddate';
  define SAS_internal / display 'Stored in SAS';
  define MS_date_val / 'Microsoft Internal Value/SAS val + 21916'
         style(column)={width=2in
                        tagattr="format:mm/dd/yyyy"};
  define date / f=mmddyyd10. display 'Variable use MMDDYYD10 SAS Format';
  define SAS_date9 /display f=mmddyyp10. 'SAS_DATE9 Variable Uses MMDDYYP10 SAS Format';
run;

ods _all_ close;


title;
footnote;

** More info: ;
** For Excel, there are NO valid dates;
** which occurred before January 1, 1900.;
** If you have a date which represents an offset;
** from Jan 1, 1900, then you need to add 21,916 ;
** to make the date a SAS date offset.;
** There are 21,916 days between January 1, 1900 and January 1, 1960.; 
** December 01, 2005 is 16,771 in SAS and 38,687 in Excel. ;
** These papers and web sites explain some instances where;
** the difference is not exactly 21916 because of ;
** leap year issues.;
** (Excel treated 1900 as a leap year, but it was not a leap year).;
/*
http://www2.sas.com/proceedings/sugi29/068-29.pdf
http://www.nesug.org/proceedings/nesug06/cc/cc18.pdf
http://support.microsoft.com/kb/214326
http://support.microsoft.com/kb/214058/EN-US/
*/

